public class MyBitSet implements MySet {

	private static int SET_SIZE_BYTES=8;
	
     /* Initially assume that set size is 64 elements => 64 bits or 8 bytes */
     /* It should scale up to the maximum value added; I.e. when adding the element 254, it should have 32 bytes */
	private byte els[];
	private Integer lock;

	public MyBitSet() {
		int i;
		els=new byte[SET_SIZE_BYTES];
		for(i=0;i<els.length;++i) 
			els[i]=0;	
		lock=0;
	}

	public void addElement(int el) {
		synchronized(lock) {
			int byindex=el/8;
			int reqsize=byindex+1;
			int mask=1<<(el%8);	
			if(reqsize>els.length) { //need resize
				int i;
				byte elold[]=els;
				els=new byte[reqsize];
				for(i=0;i<elold.length;++i)
					els[i]=elold[i];
				for(;i<els.length;++i)	
					els[i]=0;
			}
			els[byindex]|=mask;		
		}
	}

	public void removeElement(int el) {
		synchronized(lock) {
			int byindex=el/8;
			int reqsize=byindex+1;
			int mask=1<<(el%8);	
			if(reqsize>=els.length) ; //Ignore
			else els[byindex]&=~mask;	
		}
	}

	public void printElements() {
		synchronized(lock) {
			int i,num=0,j,mask;
			for(i=0;i<els.length;++i) {
				mask=1;
				for(j=0;j<8;++j,++num) {
					if((els[i]&mask)!=0)
						System.out.print(num+" ");
					mask<<=1;
				}	
			}
			System.out.println();
		}
	}

	//We can NEVER do this without probably overwriting what other has.
	//One trick here is to use MySet copy=other.complementSet().complementSet();
	//Overriding other is acceptable.
	public MySet unionSet(MySet other) {
		if(other==null) return other;
		MySet copy=other.complementSet().complementSet();
		synchronized(lock) {
			int i,num=0,j,mask;
			for(i=0;i<els.length;++i) {
				mask=1;
				for(j=0;j<8;++j,++num) {
					if((els[i]&mask)!=0)
						copy.addElement(num);
					mask<<=1;
				}	
			}
		}
		return copy;
	}

	public MySet complementSet() {
		MyBitSet complement=new MyBitSet();
		synchronized(lock) {
			int i,num=0,j,mask;
			for(i=0;i<els.length;++i) {
				mask=1;
				for(j=0;j<8;++j,++num) {
					if((els[i]&mask)==0)
						complement.addElement(num);
					mask<<=1;
				}
			}
		}
		return complement;	
	}

	//We can use DeMorgan's law to to achieve this. 
	public MySet intersectionSet(MySet other) {
		return other.complementSet().unionSet(complementSet()).complementSet();
	}

	//Driver Program.
	public static void main(String []args) {
		MySet s1=new MyBitSet();
		MySet s2=new MyBitSet();
		s1.addElement(64);
		s1.addElement(43);
		s1.addElement(3);
		s1.addElement(2);
		s2.addElement(67);
		s2.addElement(3);
		s2.addElement(2);
		s2.addElement(1);
		s1.printElements();
		s2.removeElement(1);
		s2.printElements();
		s1.unionSet(s2).printElements();
		s1.intersectionSet(s2).printElements();
		s1.complementSet().printElements();
	}
}

